package at.technikumwien.movies;

import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import java.net.URI;
import java.util.List;

@Path("/actor")
public class ActorResource {
    @Inject
    private ActorsService actorsService;

    @Context
    private UriInfo uriInfo;

    @GET
    @Produces({
            MediaType.APPLICATION_JSON,
            MediaType.APPLICATION_XML
    })
    @RolesAllowed("MSRead")
    public List<Actor> retrieveAll() {
        return actorsService.findAllActors();
    }

    @GET
    @Produces({
            MediaType.APPLICATION_JSON,
            MediaType.APPLICATION_XML
    })
    @Path("/{id}")
    @RolesAllowed("MSRead")
    public Actor retrieve(@PathParam("id") long id) {
        return actorsService.findActorById(id);
    }

    @DELETE
    @Path("/{id}")
    @RolesAllowed("MSWrite")
    public void delete(@PathParam("id") long id) {
        actorsService.removeActorById(id);
    }

    @POST
    @Consumes({
            MediaType.APPLICATION_JSON,
            MediaType.APPLICATION_XML
    })
    @RolesAllowed("MSWrite")
    public Response create(Actor actor) {
        actor.setId(null); // Make sure that a new actor is added, not overwriting existing one
        List<Actor> newActors = actorsService.saveActor(List.of(actor));

        URI location = uriInfo.getAbsolutePathBuilder().path(newActors.get(0).getId().toString()).build();

        return Response.created(location).build();
    }

    @PUT
    @Consumes({
            MediaType.APPLICATION_JSON,
            MediaType.APPLICATION_XML
    })
    @RolesAllowed("MSWrite")
    @Path("/{id}")
    public void update(@PathParam("id") long id, Actor actor) {
        actor.setId(id);
        actorsService.saveActor(List.of(actor));
    }
}
