package at.technikumwien.movies;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.*;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import javax.security.enterprise.SecurityContext;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

@Stateless
@TransactionManagement(value=TransactionManagementType.CONTAINER)
public class ActorsService {
    private static final Logger LOGGER = Logger.getLogger(ActorsService.class.getName());

    @Resource
    private SessionContext context;

    @PersistenceContext(unitName = "MoviesPU")
    private EntityManager em;

    @Inject
    private SecurityContext securityContext;

    public Actor findActorById(long id) {
        LOGGER.info("findActorById() >> id=" + id);

        Actor actor = em.find(Actor.class, id);
        if (actor == null) {
            throw new EntityNotFoundException("can't find actor with id=" + id);
        }

        return actor;
    }

    public List<Actor> findAllActors() {
        LOGGER.info("findAllActors)");

        return em.createNamedQuery("Actors.selectAllActors", Actor.class)
                .getResultList();
    }

    public void removeActorById(long id) {
        LOGGER.info("removeActorById() >> id=" + id);

        Actor actor = findActorById(id);
        em.remove(actor);
    }

    // TODO maybe check if the actor already exists?
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public List<Actor> saveActor(List<Actor> actors) {

        try {
            List<Actor> addedActors = new ArrayList<>();

            LOGGER.info("save() >> actors" + actors);
            for (Actor actorToImport : actors) {
                addedActors.add(em.merge(actorToImport));
            }

            return addedActors;
        } catch (Exception e) {
            LOGGER.info("Rollback when adding actor!");
            context.setRollbackOnly();
            e.printStackTrace();
        }

        return null;
    }
}